@@ -13,7 +13,7 @@ from cryptography.x509.name import _ASN1Type
 from clairmeta.settings import DCP_SETTINGS
 from clairmeta.utils.xml import canonicalize_xml
 from clairmeta.utils.sys import all_keys_in_dict
-from clairmeta.dcp_check import CheckerBase, CheckException
+from clairmeta.dcp_check import CheckerBase
 
 
 class Checker(CheckerBase):
@@ -23,8 +23,8 @@ class Checker(CheckerBase):
         2006 : D-Cinema Operations - Digital Certificate, section 6.2.
     """
 
-    def __init__(self, dcp, profile):
-        super(Checker, self).__init__(dcp, profile)
+    def __init__(self, dcp):
+        super(Checker, self).__init__(dcp)
 
         self.init_context()
 
@@ -72,7 +72,7 @@ class Checker(CheckerBase):
     def issuer_to_str(self, issuer):
         """ String representation of X509Name object. """
         # Note : what are the escapes rule here ?
-        issuer_dn = issuer.dnQualifier.replace('+', '\+')
+        issuer_dn = issuer.dnQualifier.replace('+', r'\+')
         return "dnQualifier={},CN={},OU={},O={}".format(
             issuer_dn, issuer.CN, issuer.OU, issuer.O)
 
@@ -89,14 +89,13 @@ class Checker(CheckerBase):
         return issuers[0] == issuers[1]
 
     def certif_der_decoding(self, cert):
-        # 1. ASN.1 DER decoding rules
+        """ Certificate ASN.1 DER decoding. """
         try:
             certif = base64.b64decode(cert['X509Certificate'])
             X509 = crypto.load_certificate(crypto.FILETYPE_ASN1, certif)
             return X509
-        except (crypto.Error) as e:
-            raise CheckException("Invalid certificate encoding : {}".format(
-                str(e)))
+        except crypto.Error as e:
+            self.error("Invalid certificate encoding : {}".format(str(e)))
 
     def certif_ext_map(self, cert):
         extensions_map = {}
@@ -127,7 +126,10 @@ class Checker(CheckerBase):
             self.cert_chains = source_xml['Signature']['KeyInfo']['X509Data']
 
             for index, cert in reversed(list(enumerate(self.cert_chains))):
-                cert_x509 = self.certif_der_decoding(cert)
+                cert_x509 = self.run_check(self.certif_der_decoding, cert)
+                if not cert_x509:
+                    continue
+
                 self.cert_store.add_cert(cert_x509)
                 self.cert_list.append(cert_x509)
 
@@ -140,6 +142,9 @@ class Checker(CheckerBase):
                 [self.run_check(check, cert_x509, cert, stack=stack)
                  for check in self.find_check('xml_certif')]
 
+            if not self.cert_list:
+                return self.checks
+
             checks = self.find_check('sign')
             [self.run_check(check, source_xml, stack=asset_stack)
              for check in checks]
@@ -154,17 +159,17 @@ class Checker(CheckerBase):
     def check_certif_version(self, cert, index):
         """ Certificate version check (X509 v3).
 
-            Reference :
-                SMPTE 430-2-2017 6.2 2
+            References:
+                SMPTE ST 430-2:2017 6.2 2
         """
         if cert.get_version() != crypto.x509.Version.v3.value:
-            raise CheckException("Invalid certificate version")
+            self.error("Invalid certificate version")
 
     def check_certif_extensions(self, cert, index):
         """ Certificate mandatory extensions check.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 3
+            References:
+                SMPTE ST 430-2:2017 6.2 3
         """
         extensions_map = self.certif_ext_map(cert)
         required_extensions = [
@@ -177,7 +182,7 @@ class Checker(CheckerBase):
         # 3.a Required extensions are present
         for ext_name in required_extensions:
             if ext_name not in extensions_map:
-                raise CheckException(
+                self.error(
                     "Missing required extension marked : {}".format(ext_name))
 
         # 3.b Unknown extensions marked critical
@@ -185,14 +190,14 @@ class Checker(CheckerBase):
             is_known = ext_name in required_extensions
             is_critical = ext.get_critical() != 0
             if not is_known and is_critical:
-                raise CheckException("Unknown extension marked as critical : "
-                                     "{}".format(ext_name))
+                self.error("Unknown extension marked as critical : "
+                           "{}".format(ext_name))
 
     def check_certif_fields(self, cert, index):
         """ Certificate mandatory fields check.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 4
+            References:
+                SMPTE ST 430-2:2017 6.2 4
         """
         # 4. Missing required fields
         # Fields : Non signed part
@@ -201,17 +206,17 @@ class Checker(CheckerBase):
         # Version SerialNumber Signature Issuer Subject Validity
         # SubjectPublicKeyInfo AuthorityKeyIdentifier KeyUsage BasicConstraint
         if not isinstance(cert.get_issuer(), crypto.X509Name):
-            raise CheckException("Missing Issuer field")
+            self.error("Missing Issuer field")
         if not isinstance(cert.get_subject(), crypto.X509Name):
-            raise CheckException("Missing Subject field")
+            self.error("Missing Subject field")
 
     def check_certif_fields_encoding(self, cert, index):
         """ Certificate Issuer and Subject attributes encoding check.
 
             Dn, O, OU and CN fields shall be of type PrintableString.
 
-            Reference :
-                SMPTE 430-2-2017 5.3.1, 5.3.2, 5.3.3, 5.3.4
+            References:
+                SMPTE ST 430-2:2017 5.3.1, 5.3.2, 5.3.3, 5.3.4
         """
         cert = cert.to_cryptography()
         fields = {
@@ -223,15 +228,15 @@ class Checker(CheckerBase):
             for a in field:
                 if a._type != _ASN1Type.PrintableString:
                     type_str = str(a._type).split('.')[-1]
-                    raise CheckException(
+                    self.error(
                         "{} {} field encoding should be PrintableString"
                         ", got {}".format(name, a.oid._name, type_str))
 
     def check_certif_basic_constraint(self, cert, index):
         """ Certificate basic constraint check.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 5
+            References:
+                SMPTE ST 430-2:2017 6.2 5
         """
         # 5. Check BasicConstraint
         extensions_map = self.certif_ext_map(cert)
@@ -241,19 +246,19 @@ class Checker(CheckerBase):
         is_leaf = not is_ca
 
         if re.search('CA:TRUE', bc) and is_leaf:
-            raise CheckException("CA True in leaf certificate")
+            self.error("CA True in leaf certificate")
         if re.search('CA:FALSE', bc) and is_ca:
-            raise CheckException("CA False in authority certificate")
+            self.error("CA False in authority certificate")
         if re.search('CA:TRUE', bc) and not re.search(r'pathlen:\d+', bc):
-            raise CheckException("CA True and Pathlen absent or not >= 0")
+            self.error("CA True and Pathlen absent or not >= 0")
         if re.search('CA:FALSE', bc) and re.search(r'pathlen:[^0]', bc):
-            raise CheckException("CA False and Pathlen present or non-zero")
+            self.error("CA False and Pathlen present or non-zero")
 
     def check_certif_key_usage(self, cert, index):
         """ Certificate key usage check.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 6
+            References:
+                SMPTE ST 430-2:2017 6.2 6
         """
         # 6. Check KeyUsage
         extensions_map = self.certif_ext_map(cert)
@@ -275,32 +280,31 @@ class Checker(CheckerBase):
             illegal_keys = [k for k in keys if k not in authorized_keys]
 
         if missing_keys:
-            raise CheckException("Missing flags in KeyUsage : {}".format(
+            self.error("Missing flags in KeyUsage : {}".format(
                 ', '.join(missing_keys)))
         if illegal_keys:
-            raise CheckException("Illegal flags in KeyUsage : {}".format(
+            self.error("Illegal flags in KeyUsage : {}".format(
                 ', '.join(illegal_keys)))
 
     def check_certif_organization_name(self, cert, index):
         """ Certificate organization name check.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 7
+            References:
+                SMPTE ST 430-2:2017 6.2 7
         """
         # 7. Check OrganizationName
         if cert.get_issuer().O == '':
-            raise CheckException("Missing OrganizationName in Issuer name")
+            self.error("Missing OrganizationName in Issuer name")
         if cert.get_subject().O == '':
-            raise CheckException("Missing OrganizationName in Subject name")
+            self.error("Missing OrganizationName in Subject name")
         if cert.get_subject().O != cert.get_issuer().O:
-            raise CheckException(
-                "OrganizationName mismatch for Issuer and Subject")
+            self.error("OrganizationName mismatch for Issuer and Subject")
 
     def check_certif_role(self, cert, index):
         """ Certificate role check.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 8
+            References:
+                SMPTE ST 430-2:2017 6.2 8
         """
         # 8. Check Role
         cn = cert.get_subject().CN
@@ -312,14 +316,12 @@ class Checker(CheckerBase):
 
         if is_leaf and self.dcp.schema == 'SMPTE':
             if not roles:
-                raise CheckException(
-                    "Missing role in CommonName ({})".format(cn))
+                self.error("Missing role in CommonName ({})".format(cn))
             if self.context_role not in roles:
-                raise CheckException(
-                    "Expecting {} role in CommonName ({})"
-                    .format(self.context_role, cn))
+                self.error("Expecting {} role in CommonName ({})".format(
+                    self.context_role, cn))
         if is_ca and roles:
-            raise CheckException(
+            self.error(
                 "Role(s) found in authority certificate CommonName ({})"
                 .format(cn))
 
@@ -328,8 +330,8 @@ class Checker(CheckerBase):
 
             See https://github.com/wolfgangw/backports/issues/80
 
-            Reference :
-                DCI Spec 1.3 9.4.3.5
+            References:
+                DCI DCSS (v1.3) 9.4.3.5
         """
         cn = cert.get_subject().CN
         roles_str = cn.split('.', 1)[0]
@@ -340,14 +342,14 @@ class Checker(CheckerBase):
 
         if is_leaf and self.dcp.schema == 'SMPTE':
             if roles and len(roles) > 1:
-                raise CheckException(
+                self.error(
                     "Superfluous roles found in CommonName ({})".format(cn))
 
     def check_certif_date(self, cert, index):
         """ Certificate date validation.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 9
+            References:
+                SMPTE ST 430-2:2017 6.2 9
         """
         # 9. Check time validity
         # Note : Date are formatted in ASN.1 Time YYYYMMDDhhmmssZ
@@ -365,29 +367,27 @@ class Checker(CheckerBase):
             not_after = datetime.strptime(not_after_str, '%Y%m%d%H%M%SZ')
 
             if validity_time < not_before or validity_time > not_after:
-                raise CheckException("Certificate is not valid at this time")
+                self.error("Certificate is not valid at this time")
 
     def check_certif_signature_algorithm(self, cert, index):
         """ Certificate signature algorithm check.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 10
+            References:
+                SMPTE ST 430-2:2017 6.2 10
         """
         # 10. Signature Algorithm
         signature_algorithm = cert.get_signature_algorithm().decode("utf-8")
         expected = self.certif_sig_algorithm_map[self.dcp.schema]
 
         if signature_algorithm not in expected:
-            raise CheckException(
-                "Invalid Signature Algorithm, expected {} but got {}".format(
-                    expected, signature_algorithm))
+            self.error("Invalid Signature Algorithm, expected {} but got {}"
+                       .format(expected, signature_algorithm))
 
     def check_certif_rsa_validity(self, cert, index):
         """ Certificate characteristics (RSA 2048, 65537 exp) check.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 11
-                SMPTE 430-2-2017 5.2
+            References:
+                SMPTE ST 430-2:2017 5.2, 6.2 11
         """
         # 11. Subject's PublicKey RSA validity
         expected_type = crypto.TYPE_RSA
@@ -399,35 +399,34 @@ class Checker(CheckerBase):
         key_exp = cert.get_pubkey().to_cryptography_key().public_numbers().e
 
         if key_type != expected_type:
-            raise CheckException("Subject's public key shall be an RSA key")
+            self.error("Subject's public key shall be an RSA key")
         if key_size != expected_size:
-            raise CheckException(
+            self.error(
                 "Subject's public key invalid size, expected {} but got {}"
-                "".format(expected_size, key_size))
+                .format(expected_size, key_size))
         if key_exp != expected_exp:
-            raise CheckException(
-                "Subject's public key invalid public exponent, \
-                 expected {} but got {}".format(
-                    expected_exp, key_exp))
+            self.error(
+                "Subject's public key invalid public exponent,"
+                " expected {} but got {}".format(expected_exp, key_exp))
 
     def check_certif_revokation_list(self, cert, index):
         """ Certificate revokation list check.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 12
+            References:
+                SMPTE ST 430-2:2017 6.2 12
         """
         # 12. Revokation list check
         # - Subject public key
         # - Issuer or certificate serial number
         if (self.context_revoked_certificates_id or
                 self.context_revoked_public_keys):
-            raise CheckException("Revokation list check not implemented")
+            self.error("Revokation list check not implemented")
 
     def check_certif_publickey_thumbprint(self, cert, index):
         """ Certificate public key thumbprint check.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 13
+            References:
+                SMPTE ST 430-2:2017 6.2 13
         """
         # 13. Subject's public key thumb print match dnQualifier
         dn_thumbprint = cert.get_subject().dnQualifier.encode("utf-8")
@@ -437,11 +436,10 @@ class Checker(CheckerBase):
         key_thumbprint = base64.b64encode(hashlib.sha1(key_bits).digest())
 
         if not dn_thumbprint:
-            raise CheckException("dnQualifier must be present")
+            self.error("dnQualifier must be present")
         if dn_thumbprint != key_thumbprint:
-            raise CheckException(
-                "dnQualifier mismatch, expected {} but got {}".format(
-                    key_thumbprint, dn_thumbprint))
+            self.error("dnQualifier mismatch, expected {} but got {}".format(
+                key_thumbprint, dn_thumbprint))
 
     # def check_certif_authority(self, cert, index):
     #     # 14. AuthorityKeyIdentifier
@@ -452,8 +450,8 @@ class Checker(CheckerBase):
     def check_certif_signature(self, cert, index):
         """ Certificate signature check.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 15
+            References:
+                SMPTE ST 430-2:2017 6.2 15
         """
         # 15. Validate signature using local issuer
         # Note : use openssl StoreContext object which should do this plus a
@@ -462,61 +460,58 @@ class Checker(CheckerBase):
             store_ctx = crypto.X509StoreContext(self.cert_store, cert)
             store_ctx.verify_certificate()
         except crypto.X509StoreContextError as e:
-            raise CheckException(
+            self.error(
                 "Certificate signature check failure : {}".format(str(e)))
 
     def check_xml_certif_serial_coherence(self, cert, xml_cert):
         """ XML / Certificate serial number coherence.
 
-            Reference : N/A
+            References: N/A
         """
         # i. Serial number check
         xml_serial = xml_cert['X509IssuerSerial']['X509SerialNumber']
         if xml_serial != cert.get_serial_number():
-            raise CheckException(
-                "Serial number mismatch, expected {} but got {}".format(
-                    cert.get_serial_number(), xml_serial))
+            self.error("Serial number mismatch, expected {} but got {}".format(
+                cert.get_serial_number(), xml_serial))
 
     def check_xml_certif_issuer_coherence(self, cert, xml_cert):
         """ XML / Certificate Issuer coherence.
 
-            Reference : N/A
+            References: N/A
         """
         # ii. Issuer name check
         xml_issuer = xml_cert['X509IssuerSerial']['X509IssuerName']
         issuer_str = self.issuer_to_str(cert.get_issuer())
         if not self.issuer_match(xml_issuer, issuer_str):
-            raise CheckException(
-                "IssuerName mismatch, expected {} but got {}".format(
-                    issuer_str, xml_issuer))
+            self.error("IssuerName mismatch, expected {} but got {}".format(
+                issuer_str, xml_issuer))
 
     def check_sign_chain_length(self, source):
         """ Certificates minimum chain length.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 16
+            References:
+                SMPTE ST 430-2:2017 6.2 16
         """
         # 16. Chain length
         if (self.context_chain_length and
                 len(self.cert_chains) < self.context_chain_length):
-            raise CheckException(
-                "Certificate chain length should be at least {} long, \
-                 got {}".format(
+            self.error(
+                "Certificate chain length should be at least {} long,"
+                " got {}".format(
                     self.context_chain_length, len(self.cert_chains)))
 
     def check_sign_chain_coherence(self, source):
         """ Certificates chain coherence.
 
-            Reference :
-                SMPTE 430-2-2017 6.2 17, 18, 19
+            References:
+                SMPTE ST 430-2:2017 6.2 17, 18, 19
         """
         for index in range(1, len(self.cert_list)):
             parent, child = self.cert_list[index-1], self.cert_list[index]
 
             # 17. Child Issuer match parent Subject
             if child.get_issuer() != parent.get_subject():
-                raise CheckException(
-                    "Certificate chain issuer / subject mismatch")
+                self.error("Certificate chain issuer / subject mismatch")
 
             # 18. Validity date of child contained in parent date
             child_A = datetime.strptime(
@@ -529,37 +524,37 @@ class Checker(CheckerBase):
                 parent.get_notAfter().decode("utf-8"), '%Y%m%d%H%M%SZ')
 
             if child_A < parent_A:
-                raise CheckException(
+                self.error(
                     "Start date of the child certificate shall be \
                     identical to or later than the start date of the parent \
                     certificate")
 
             if child_B > parent_B:
-                raise CheckException(
+                self.error(
                     "End date of the child certificate shall be \
                     identical to or earlier than the end date of the parent \
                     certificate")
 
             # 19. Root certificate shall appear in trusted certificate list
             if self.context_trusted_certificates:
-                raise CheckException("Trusted list check not implemented")
+                self.error("Trusted list check not implemented")
 
     def check_sign_chain_coherence_signature_algorithm(self, source):
         """ Certificates chain coherence.
 
-            Reference : N/A
+            References: N/A
         """
         sign_alg_set = set(
             [c.get_signature_algorithm() for c in self.cert_list])
         if len(sign_alg_set) > 1:
-            raise CheckException(
+            self.error(
                 "Certificate chain contains certificates "
                 "signed with different algorithm")
 
     def check_sign_signature_algorithm(self, source):
         """ XML signature algorithm check.
 
-            Reference : N/A
+            References: N/A
         """
         # Additionnal. XML coherence checks
         signed_info = source['Signature']['SignedInfo']
@@ -567,72 +562,72 @@ class Checker(CheckerBase):
         # Signature algorithm
         sig = signed_info['SignatureMethod@Algorithm']
         if self.sig_ns_map[self.dcp.schema] != sig:
-            raise CheckException(
+            self.error(
                 "Invalid Signature Algorithm, expected {} but got {}".format(
                     self.sig_ns_map[self.dcp.schema], sig))
 
     def check_sign_canonicalization_algorithm(self, source):
         """ XML canonicalization algorithm check.
 
-            Reference : N/A
+            References: N/A
         """
         signed_info = source['Signature']['SignedInfo']
         # Canonicalization algorithm
         can = signed_info['CanonicalizationMethod@Algorithm']
         if can != DCP_SETTINGS['xmluri']['c14n']:
-            raise CheckException("Invalid canonicalization method")
+            self.error("Invalid canonicalization method")
 
     def check_sign_transform_algorithm(self, source):
         """ XML signature transform algorithm check.
 
-            Reference : N/A
+            References: N/A
         """
         signed_info = source['Signature']['SignedInfo']
         # Transform alogrithm
         trans = signed_info['Reference']['Transforms']['Transform@Algorithm']
         if trans != DCP_SETTINGS['xmluri']['enveloped_sig']:
-            raise CheckException("Invalid transform method")
+            self.error("Invalid transform method")
 
     def check_sign_digest_algorithm(self, source):
         """ XML signature digest method check.
 
-            Reference : N/A
+            References: N/A
         """
         signed_info = source['Signature']['SignedInfo']
         # Digest algorithm
         trans = signed_info['Reference']['DigestMethod@Algorithm']
         if trans != DCP_SETTINGS['xmluri']['sha1']:
-            raise CheckException("Invalid digest method")
+            self.error("Invalid digest method")
 
     def check_sign_issuer_name(self, source):
         """ XML signature issuer name check.
 
-            Reference : N/A
+            References: N/A
         """
         signer = source['Signer']['X509Data']['X509IssuerSerial']
         # Signer Issuer Name
         issuer_dn = self.issuer_to_str(self.cert_list[-1].get_issuer())
         if not self.issuer_match(signer['X509IssuerName'], issuer_dn):
-            raise CheckException("Invalid Signer Issuer Name")
+            self.error("Invalid Signer Issuer Name")
 
     def check_sign_issuer_serial(self, source):
         """ XML signature serial number check.
 
-            Reference : N/A
+            References: N/A
         """
         sig = source['Signer']['X509Data']['X509IssuerSerial']
         # Signer Serial number
         if sig['X509SerialNumber'] != self.cert_list[-1].get_serial_number():
-            raise CheckException("Invalid Signer Serial Number")
+            self.error("Invalid Signer Serial Number")
 
     def check_document_signature(self, source, path):
         """ Digital signature validation.
 
-            Reference :
-                SMPTE 429-7-2006 6.13
-                SMPTE 429-8-2007 5.10
-                RFC 3275 https://www.ietf.org/rfc/rfc3275.txt
-                RFC 4051 https://tools.ietf.org/html/rfc4051
+            References:
+                SMPTE ST 429-7:2006 6.13
+                SMPTE ST 429-8:2007 5.10
+                IETF RFC 3275
+                IETF RFC 4051
         """
         # Check digest (XML document hash)
         signed_info = source['Signature']['SignedInfo']
@@ -645,8 +640,7 @@ class Checker(CheckerBase):
         c14n_digest = base64.b64encode(self.digest_func(c14n_doc).digest())
         c14n_digest = c14n_digest.decode("utf-8")
         if xml_digest != c14n_digest:
-            raise CheckException(
-                "XML Digest mismatch, signature can't be checked")
+            self.error("XML Digest mismatch, signature can't be checked")
 
         # Check signature (XML document hash encrypted with certifier
         # private key)
@@ -665,4 +659,4 @@ class Checker(CheckerBase):
                 c14n_sign,
                 self.sig_algorithm_map[self.dcp.schema])
         except crypto.Error as e:
-            raise CheckException("Signature validation failed")
+            self.error("Signature validation failed")
